package com.yixin.ms.server;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.Maps;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.server.dto.JiliangCanPassDTO;
import com.yixin.ms.server.dto.JiliangCanPassDataDTO;
import com.yixin.ms.server.vo.HealthResultVO;
import com.yixin.ms.server.vo.JiliangCanPassVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * @author: lileilei
 * @date: 2022/8/25 15:53
 * @description: 计量大学校园平台开放接口服务
 */
@Slf4j
@Component
public class JiliangUniversityServer {
    /**
     * token 缓存标识
     */
    private static final String TOKEN = "jiliang_token";

    @Value("${jiliang.appKey}")
    private String appKey;
    @Value("${jiliang.appSecret}")
    private String appSecret;
    @Value("${jiliang.getTokenUrl}")
    private String getTokenUrl;
    @Value("${jiliang.canPassUrl}")
    private String canPassUrl;

    /**
     * 请求密钥有效时间为 一天 12H
     *
     * @see <a href="https://www.hutool.cn/docs/#/cache/TimedCache">定时缓存</a>
     */
    private static TimedCache<String, String> tokenCache = CacheUtil.newTimedCache(12 * 60 * 60 * 1000);

    /**
     * 获取accesskey（有效期一天）
     * @return
     */
    public String getToken(){
        String token = tokenCache.get(TOKEN, false);
        if(StrUtil.isNotEmpty(token)){
            return token;
        }

        Map<String, Object> paramMap = MapUtil.newHashMap();
        paramMap.put("appKey", appKey);
        paramMap.put("appSecret", appSecret);
        HealthResultVO resultVO = null;
        try {
            String body = HttpUtil.post(getTokenUrl, paramMap);
            log.info("计量大学开放平台获取token响应内容：{}", body);
            resultVO = JSON.parseObject(body, HealthResultVO.class);
            if (resultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                tokenCache.put(TOKEN, resultVO.getData().toString());
            }
        }catch (Exception e){
            throw new ServiceException("学校开放接口服务异常，请联系工作人员");
        }

        return resultVO.getData().toString();
    }

    /**
     * 获取通过结果 0-不通过，1-通过
     * @param dto
     * @return
     */
    public String canPass(JiliangCanPassDataDTO dto){
        Map<String, String> headers = Maps.newHashMap();
        headers.put("x-access-token", this.getToken());
        HealthResultVO<JiliangCanPassVO> resultVO = null;
        try {
            String body = HttpUtil.createPost(canPassUrl).addHeaders(headers).body(JSON.toJSONString(new JiliangCanPassDTO().setData(dto))).execute().body();
            log.info("计量大学开放平台获取通过结果响应内容：{}", body);
            resultVO = JSON.parseObject(body, HealthResultVO.class);
            if (resultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                resultVO.setData(JSON.parseObject(JSON.toJSONString(resultVO.getData()), JiliangCanPassVO.class));
            }
        }catch (Exception e){
            throw new ServiceException("学校开放接口服务异常，请联系工作人员");
        }

        return resultVO.getData().getCanPass();
    }
}
